﻿Imports System.IO
Imports System.Runtime.InteropServices

Public Class FormMain
    Private Const serviceGmail As Integer = 0
    Private Const serviceOffice365 As Integer = 1
    Private ReadOnly AuthUri() As String = {
        "https://accounts.google.com/o/oauth2/auth",
        "https://login.microsoftonline.com/{0}/oauth2/v2.0/authorize"
    }
    Private ReadOnly AuthScope() As String = {
        "https://mail.google.com/",
        "offline_access https://outlook.office.com/SMTP.Send https://outlook.office.com/POP.AccessAsUser.All https://outlook.office.com/IMAP.AccessAsUser.All"
    }
    Private ReadOnly AuthParam() As String = {
        "access_type=offline",
        ""
    }
    Private ReadOnly TokenUri() As String = {
        "https://accounts.google.com/o/oauth2/token",
        "https://login.microsoftonline.com/{0}/oauth2/v2.0/token"
    }
    Private ReadOnly Pop3Host() As String = {
        "pop.gmail.com",
        "Outlook.office365.com"
    }
    Private ReadOnly Imap4Host() As String = {
        "imap.gmail.com",
        "Outlook.office365.com"
    }
    Private ReadOnly SmtpHost() As String = {
        "smtp.gmail.com",
        "Smtp.office365.com"
    }
    Public Class OAuthParameter
        Public ServiceType As Integer
        Public ClientId As String
        Public ClientSecret As String
        Public TenantId As String
        Public RedirectUri As String
        Public RefreshToken As String
        Public AccessToken As String
        Public MailAddress As String
    End Class
    Private Const XmlFileName As String = "vboauthparam.xml"   ' 設定保存ファイル
    Private OAuthParam As OAuthParameter                  ' メールサーバ等の設定
    Private Oauth As nMail.OAuth2                         ' nMail.DLL OAuth2 クラス
    Private StartFlag As Boolean = False
    Private Title As String = "vboauth"
    Private SubjectReceived As String
    Private FromReceived As String

    Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        OAuthParam = New OAuthParameter
        If File.Exists(XmlFileName) Then
            Dim serializer As System.Xml.Serialization.XmlSerializer = New System.Xml.Serialization.XmlSerializer(GetType(OAuthParameter))
            Dim fs As FileStream = New FileStream(XmlFileName, FileMode.Open)
            OAuthParam = CType(serializer.Deserialize(fs), OAuthParameter)
            fs.Close()
        End If
        nMail.Winsock.Initialize()

        ComboService.SelectedIndex = OAuthParam.ServiceType
        TextClientId.Text = OAuthParam.ClientId
        TextClientSecret.Text = OAuthParam.ClientSecret
        TextTenantId.Text = OAuthParam.TenantId
        TextRedirectUri.Text = OAuthParam.RedirectUri
        TextRefreshToken.Text = OAuthParam.RefreshToken
        TextAccessToken.Text = OAuthParam.AccessToken
        TextMailAddress.Text = OAuthParam.MailAddress
        CheckEnableControl()

        ' 認可コード取得イベント設定
        Oauth = New nMail.OAuth2
        AddHandler Oauth.AuthorizeEvent, AddressOf AuthenticateEvent
    End Sub

    Private Sub FormMain_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        nMail.Winsock.Done()
        WriteXml()
    End Sub

    Private Sub ComboService_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboService.SelectedIndexChanged
        CheckEnableControl()
    End Sub

    Private Sub ButtonGetRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonGetRefresh.Click
        If StartFlag Then
            Oauth.EndAuthorize()
            StartFlag = False
            ButtonGetRefresh.Text = "リフレッシュトークン取得"
        Else
            Oauth.AuthUri = LabelAuthUri.Text
            Oauth.TokenUri = labelTokenUri.Text
            Oauth.ClientId = textClientId.Text
            Oauth.ClientSecret = textClientSecret.Text
            Oauth.RedirectUri = textRedirectUri.Text
            Oauth.ResponseOk = "<html><head><meta charset=""utf-8""><title>取得成功</title></head><body>認可コードの取得成功</body></html>"
            Oauth.ResponseError = "<html><head><meta charset=""utf-8""><title>取得エラー</title></head><body>認可コードの取得エラー</body></html>"
            Oauth.Scope = AuthScope(comboService.SelectedIndex)
            Oauth.AuthParameter = AuthParam(comboService.SelectedIndex)
            Oauth.StateEnable = True
            textRefreshToken.Text = ""
            textAccessToken.Text = ""
            Try
                Oauth.StartAuthorize()
                StartFlag = True
                ButtonGetRefresh.Text = "中止"
            Catch fex As FormatException
                MessageBox.Show("パラメータを入力してください。", Title)
            Catch iex As InvalidOperationException
                MessageBox.Show("サーバーが起動できません。" & vbCrLf & "リダイレクトURIのポート番号を確認してください。", Title)
            Catch ex As Exception
                Debug.WriteLine(ex.Message)
            End Try
        End If
    End Sub

    Private Sub ButtonGetAccess_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonGetAccess.Click
        Try
            Oauth.TokenUri = labelTokenUri.Text
            Oauth.ClientId = textClientId.Text
            Oauth.ClientSecret = textClientSecret.Text
            Oauth.RefreshToken = textRefreshToken.Text
            Oauth.GetAccessToken()
            textAccessToken.Text = Oauth.AccessToken
            Clipboard.SetText(textAccessToken.Text)
        Catch fex As FormatException
            MessageBox.Show("パラメータを入力してください。", Title)
        Catch iex As InvalidOperationException
            MessageBox.Show("アクセストークンの取得に失敗しました。", Title)
        Catch nex As nMail.nMailException
            MessageBox.Show(String.Format("エラーが発生しました。{0}", nex.Message), Title)
        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try
    End Sub

    Private Sub WriteXml()
        OAuthParam.ServiceType = ComboService.SelectedIndex
        OAuthParam.ClientId = TextClientId.Text
        OAuthParam.ClientSecret = TextClientSecret.Text
        OAuthParam.TenantId = TextTenantId.Text
        OAuthParam.RedirectUri = TextRedirectUri.Text
        OAuthParam.RefreshToken = TextRefreshToken.Text
        OAuthParam.AccessToken = TextAccessToken.Text
        OAuthParam.MailAddress = TextMailAddress.Text

        Dim serializer As System.Xml.Serialization.XmlSerializer = New System.Xml.Serialization.XmlSerializer(GetType(OAuthParameter))
        Dim fs As FileStream = New FileStream(XmlFileName, FileMode.Create)
        serializer.Serialize(fs, OAuthParam)
        fs.Close()
    End Sub

    Private Sub CheckEnableControl()
        If ComboService.SelectedIndex = 1 Then
            TextTenantId.Enabled = True
        Else
            TextTenantId.Enabled = False
        End If
        LabelAuthUri.Text = String.Format(AuthUri(ComboService.SelectedIndex), TextTenantId.Text)
        LabelTokenUri.Text = String.Format(TokenUri(ComboService.SelectedIndex), TextTenantId.Text)
    End Sub

    Private Delegate Sub AuthenticateEventDelegate(ByVal sender As Object, ByVal e As System.EventArgs)

    Private Sub AuthenticateEvent(ByVal sender As Object, ByVal e As System.EventArgs)
        If InvokeRequired Then
            BeginInvoke(New AuthenticateEventDelegate(AddressOf AuthenticateEvent), New Object() {sender, e})
        Else
            Try
                Oauth.GetRefreshToken()
                TextRefreshToken.Text = Oauth.RefreshToken
                TextAccessToken.Text = Oauth.AccessToken
                Clipboard.SetText(TextRefreshToken.Text)
            Catch fex As FormatException
                MessageBox.Show("パラメータを入力してください。", Title)
            Catch iex As InvalidOperationException
                MessageBox.Show("リフレッシュトークンの取得に失敗しました。" & vbCrLf & "トークン取得URIが正しいか確認してください。", Title)
            Catch nex As nMail.nMailException
                If nex.ErrorCode = nMail.OAuth2.ErrorNoToken Then
                    MessageBox.Show("リフレッシュトークンの取得に失敗しました。" & vbCrLf & "アカウントからログアウト後、再度実行してください。", Title)
                Else
                    MessageBox.Show(String.Format("エラーが発生しました。{0}", nex.Message), Title)
                End If
            Catch ex As Exception
                Debug.WriteLine(ex.Message)
            End Try
            Oauth.EndAuthorize()
            StartFlag = False
            ButtonGetRefresh.Text = "リフレッシュトークン取得"
        End If
    End Sub

    Private Sub TextTenantId_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextTenantId.TextChanged
        CheckEnableControl()
    End Sub

    Private Sub ButtonReceive_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonReceive.Click
        If TextMailAddress.Text = "" Then
            MessageBox.Show("メールアドレスを入力してください。", Title)
            Return
        End If
        If TextAccessToken.Text = "" Then
            MessageBox.Show("アクセストークンを取得してください。", Title)
            Return
        End If
        If MessageBox.Show("メールを受信しますか？", Title, MessageBoxButtons.YesNo) = DialogResult.Yes Then
            Using pop As New nMail.Pop3()
                Try
                    nMail.Options.EnableConnectTimeout()
                    pop.SSL = nMail.Pop3.TLS1
                    pop.Port = nMail.Pop3.StandardSslPortNo
                    pop.Connect(Pop3Host(comboService.SelectedIndex))
                    pop.Authenticate(textMailAddress.Text, textAccessToken.Text, nMail.Pop3.AuthOAuth2)
                    If numericMailNo.Value <= pop.Count Then
                        pop.GetMail(numericMailNo.Value)
                        textTo.Text = pop.From
                        textSubject.Text = pop.Subject
                        textBody.Text = pop.Body
                        FromReceived = pop.From
                        SubjectReceived = pop.Subject
                    Else
                        MessageBox.Show("指定の番号のメールが存在しません。", Title)
                    End If
                    pop.Close()
                Catch nex As nMail.nMailException
                    If nex.ErrorCode = nMail.Pop3.ErrorAuthenticate Then
                        MessageBox.Show(String.Format("認証エラーです。\r\n{0}", nex.Message), Title)
                    Else
                        MessageBox.Show(String.Format("サーバー接続時にエラーが発生しました。\r\n{0}", nex.Message), Title)
                    End If
                Catch ex As Exception
                    Debug.WriteLine(ex.Message)
                End Try
            End Using
        End If
    End Sub

    Private Sub ButtonSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSend.Click
        If TextMailAddress.Text = "" Then
            MessageBox.Show("メールアドレスを入力してください。", Title)
            Return
        End If
        If TextAccessToken.Text = "" Then
            MessageBox.Show("アクセストークンを取得してください。", Title)
            Return
        End If
        If TextTo.Text = "" Then
            MessageBox.Show("宛先を入力してください", Title)
            Return
        End If
        If TextSubject.Text = "" Then
            MessageBox.Show("件名を入力してください", Title)
            Return
        End If
        If TextBody.Text = "" Then
            MessageBox.Show("本文を入力してください", Title)
            Return
        End If
        If (TextTo.Text = FromReceived) And (TextSubject.Text = SubjectReceived) Then
            MessageBox.Show("受信したメールをそのまま返送してしまいます。" & vbCrLf & "宛先や件名を変更してください。", Title)
            Return
        End If
        If MessageBox.Show("メールを送信しますか？", Title, MessageBoxButtons.YesNo) = DialogResult.Yes Then
            Using smtp As New nMail.Smtp()
                Try
                    smtp.SSL = nMail.Smtp.TLS1
                    smtp.Port = nMail.Smtp.StandardSslPortNo
                    If ComboService.SelectedIndex = serviceOffice365 Then
                        smtp.SSL = smtp.SSL Or nMail.Smtp.STARTTLS
                        smtp.Port = nMail.Smtp.SubmissionPortNo
                    End If
                    smtp.Connect(SmtpHost(ComboService.SelectedIndex))
                    smtp.Authenticate(TextMailAddress.Text, TextAccessToken.Text, nMail.Smtp.AuthOAuth2)
                    smtp.SendMail(TextTo.Text, TextMailAddress.Text, TextSubject.Text, TextBody.Text)
                    smtp.Close()

                    MessageBox.Show("送信成功しました。")
                Catch nex As nMail.nMailException
                    MessageBox.Show(String.Format("{0:d}", nex.ErrorCode))
                Catch ex As Exception
                    Debug.WriteLine(ex.Message)
                End Try
            End Using
        End If
    End Sub

    Private Sub buttonReceiveImap4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonReceiveImap4.Click
        If TextMailAddress.Text = "" Then
            MessageBox.Show("メールアドレスを入力してください。", Title)
            Return
        End If
        If TextAccessToken.Text = "" Then
            MessageBox.Show("アクセストークンを取得してください。", Title)
            Return
        End If
        If MessageBox.Show("メールを受信しますか？", Title, MessageBoxButtons.YesNo) = DialogResult.Yes Then
            Using imap As New nMail.Imap4()
                Try
                    nMail.Options.EnableConnectTimeout()
                    imap.SSL = nMail.Imap4.TLS1
                    imap.Port = nMail.Imap4.StandardSslPortNo
                    imap.Connect(Imap4Host(ComboService.SelectedIndex))
                    imap.Authenticate(TextMailAddress.Text, TextAccessToken.Text, nMail.Imap4.AuthOAuth2)
                    imap.SelectMailBox(textMailBox.Text)
                    If NumericMailNo.Value <= imap.Count Then
                        imap.GetMail(NumericMailNo.Value)
                        TextTo.Text = imap.From
                        TextSubject.Text = imap.Subject
                        TextBody.Text = imap.Body
                        FromReceived = imap.From
                        SubjectReceived = imap.Subject
                    Else
                        MessageBox.Show("指定の番号のメールが存在しません。", Title)
                    End If
                    imap.Close()
                Catch nex As nMail.nMailException
                    If nex.ErrorCode = nMail.Imap4.ErrorAuthenticate Then
                        MessageBox.Show(String.Format("認証エラーです。\r\n{0}", nex.Message), Title)
                    Else
                        MessageBox.Show(String.Format("サーバー接続時にエラーが発生しました。\r\n{0}", nex.Message), Title)
                    End If
                Catch ex As Exception
                    Debug.WriteLine(ex.Message)
                End Try
            End Using
        End If
    End Sub
End Class
